/// TINY-MPP by Holle, Lizenz: Darf jeder nutzen, egal wofür.
///                                      __  __
///                                VCC -|o \/  |- GND             
///            (PCINT8/XTAL1/CLKI) PB0 -|ATTINY|- PA0 (ADC0/AREF/PCINT0)
///                 (PCINT9/XTAL2) PB1 -| 44A  |- PA1 (ADC1/AIN0/PCINT1)   
///             (PCINT11/RESET/dW) PB3 -|      |- PA2 (ADC2/AIN1/PCINT2)   
///      (PCINT10/INT0/OC0A/CKOUT) PB2 -|      |- PA3 (ADC3/T0/PCINT3)               
///         (PCINT7/ICP/OC0B/ADC7) PA7 -|rc    |- PA4 (ADC4/USCK/SCL/T1/PCINT4)
/// (PCINT6/OC1A/SDA/MOSI/DI/ADC6) PA6 -|t__esc|- PA5 (ADC5/DO/MISO/OC1B/PCINT5)
/*
TX
+100%		xx,xV		(2200-1900=300)
 +50%		xx,xV		(2200-1700=500)
   0%		xx,xV		(2200-1500=700)
 -50%		xx,xV		(2200-1300=1000)
-100%		Motor aus
*/


#define F_CPU 8000000UL
#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>  

uint16_t MPP=1111;
uint16_t SVolt=0;
uint16_t RC=1100;
uint16_t ICR1a;
uint16_t PWM;
uint8_t  count=47;
uint8_t  csum;

int main(void){
	DDRA   |= 0b00100000;		/// ESC-PPM-OUT OC1B Attinyx4 = PA5(D5) 
	PORTA  |= 0b11000000;		/// READ-RC-IN  ICP1 Attinyx4 = PA7(D7)
	TCCR0A  = 0b00000010;   // TELEMTRIE USI-Timer in CTC mode  
	TCCR0B  = 0b00000010;   // TELEMTRIE USI-Timer prescaler 1/8
	OCR0A   = 53;           // TELEMTRIE USI-Timer OVF 52,083333333uS for 19200
	GIMSK   = 0b00010000;   // TELEMTRIE USI-Timer Interrupt
	PCMSK0  = 0b01000000;		// TELEMTRIE USI-Timer Interrupt 
	TCCR1A	= 0b00100011;		/// Fast PWM,   OC1A as Top, OC1B is PWM
	TCCR1B	= 0b01011010;		/// ICP-rising, Prescale 1/8
	TIMSK1	= 0b00100000;		/// ICP-ISR activ
	OCR1A	= 2000;					/// PPM-OUT Timer Top for 2mS 500Hz
	OCR1B	= 1050;					/// Start PPM ESC
	ADMUX	= 0b10000010;		/// Read ADC2, Aref 1,1V,
	ADCSRA	= 0b11000110;		/// Read ADC Prescale /64	
	sei();
	_delay_ms(2500);

/// --- ENDLESS LOOP --- ///
while(1){

/// --- 500Hz Sync mit PWM-OUT --- ///
while(	TIFR1	&	(1<<TOV1))					/// Auf PWM-OVR warten für 2mS Loop
{				TIFR1	|=(1<<TOV1); 					/// wenn gesetzt dann OVF1 Flag löschen.

/// --- TELEMETRIE-ROUTINE --- ///
HOTTSEND();													/// Hott Telemetrie senden
//SERIALSEND(OCR1B /8);							/// oder Serial-debug
while(TCNT1<600);   								/// Serial endet immer bei 600mS

/// --- MPP BERECHNEN AUS RC BERRECHNEN--- /// 
		if (RC < 1200)	{MPP = 1111;			} 	///	Motor aus wenn Knüppel ganz hinten
		else 			{MPP = 2200 - RC;	}		///	MPP wird Knüppelabhängig errechnet 


/// --- SPANNUNG MESSEN --- ///
SVolt=0;
for (int i=1; i <= 8; i++){
		ADCSRA |= (1<<ADSC);
		while(ADCSRA &(1<<ADSC));
		SVolt += ADC;
	}	SVolt /= 8;
	
/// --- REGELUNG --- ///
if (SVolt > MPP) {PWM = OCR1B +1 + ((SVolt - MPP) / 4); }
else						 {PWM = OCR1B -1 - ((MPP - SVolt) / 5); }
if (PWM > 1900)  {PWM = 1900; }
if (PWM < 1050)  {PWM = 1050; }
OCR1B = PWM;
}}}
/// --- ENDLESS-LOOP-END --- ///


/// + + + + + + ISR ZUM RC AUSLESEN + + + + + + + ///
ISR (TIMER1_CAPT_vect) {						/// RC READ ISR  
if(TCCR1B   &  (1<<ICES1))					/// we have rising edge -
{   TCCR1B &= ~(1<<ICES1);					/// switch ICP to falling edge                                             
    ICR1a   =  ICR1;								//  Hier Ende, Rest oben/// store ICP1 IN ICP1a
} else															/// we have falling edge  
{   TCCR1B |=  (1<<ICES1);					/// switch ICP @ rising edge                                        
 if (ICR1   <  ICR1a)								/// if Overflow ? 
      {RC   =  ICR1+2000-ICR1a;}		/// Yes > Calc RC with OVF
 else {RC   =  ICR1     -ICR1a;}		/// No  > Calc RC without OVF
}}


/// + + + + + AB HIER TELEMETRIE HOTT + + + + + +

void SERIALSEND(uint8_t SendByte){ 	    // blocking 0,5mS //
csum+=SendByte;   // Hott Checksum 
while(TCNT1<100);																								 {PORTA &= ~(1<<PA6);}
while(TCNT1<152); if(SendByte & _BV(0)){PORTA |= (1<<PA6);} else {PORTA &= ~(1<<PA6);}
while(TCNT1<204); if(SendByte & _BV(1)){PORTA |= (1<<PA6);} else {PORTA &= ~(1<<PA6);}
while(TCNT1<256); if(SendByte & _BV(2)){PORTA |= (1<<PA6);} else {PORTA &= ~(1<<PA6);}
while(TCNT1<308); if(SendByte & _BV(3)){PORTA |= (1<<PA6);} else {PORTA &= ~(1<<PA6);}
while(TCNT1<360); if(SendByte & _BV(4)){PORTA |= (1<<PA6);} else {PORTA &= ~(1<<PA6);}
while(TCNT1<412); if(SendByte & _BV(5)){PORTA |= (1<<PA6);} else {PORTA &= ~(1<<PA6);}
while(TCNT1<464); if(SendByte & _BV(6)){PORTA |= (1<<PA6);} else {PORTA &= ~(1<<PA6);}
while(TCNT1<516); if(SendByte & _BV(7)){PORTA |= (1<<PA6);} else {PORTA &= ~(1<<PA6);}
while(TCNT1<568); 							  		 {PORTA |= (1<<PA6);}
}

void HOTTSEND(){ 
if 	(count<48){
	switch(count){ 
	default:  		SERIALSEND(0x00);      		break;
	case  0:		DDRA |= (1 << PA6);			break;	/// TransmitterMode() is output//GIMSK  &= ~(1<<PCIE0);			/// Disable pin change interrupts 
	case  1:		csum  = 0b00000000; 		break;  ///  
	case  2:		/*uint16_t U=SVolt/85*/;     											break;  ///  
	case  3:		SERIALSEND(0x7C);			break;  /// StartByte
	case  4:		SERIALSEND(0x8E);			break;  /// 8E = EAM
	case  6:		SERIALSEND(0xE0);			break;  /// Sensor-ID 
	case 31:		SERIALSEND(MPP&255);		break;  // L-Byte Strom
	case 32:		SERIALSEND(MPP>>8);			break;  // H-Byte Strom
	case 33:		SERIALSEND(SVolt&255);		break;  // L-Byte Spannung
	case 34:		SERIALSEND(SVolt>>8);		break;  // H-Byte Spannung
	case 35:		SERIALSEND(OCR1BL);			break;  // L-Byte mAh
	case 36:		SERIALSEND(OCR1BH);			break;  // H-Byte mAh
	case 46:		SERIALSEND(0x7D);			break;  /// End Byte 
	case 47:		SERIALSEND(csum); 								/// Checksum + back to Rx
					DDRA  &= ~(1<<  PA6); 						/// Switch Pin to Input
					GIMSK |=  (1<<PCIE0);		break; 	/// Enable pin change interrupts again
}					count++;
}}

ISR (PCINT0_vect) { 
	if (!(PINA & 1<<PA6)){		/// fallende Flanke RX; ein Byte wird kommen  
		GIMSK &= ~(1<<PCIE0);		/// RX deaktiveren Disable pin change interrupts 
		TCNT0  = 0b11101000; 		/// 1,5= 232 bis zur ersten Messung
		USICR  = 0b01000100;		/// USI OVR aktiv, clock souce is timer0
		USISR  = 0b01101000;		/// USI Counter auf 8 und OFV Flag löschen 
}		}  

ISR (USI_OVF_vect) {        /// ein Byte wurde empfangen
	 USICR =  0b00000000;		/// USI deaktivieren  
if	(USIDR == 0b01110001)   //	8E	10001110<>01110001	71 EAM
	{count =  0b00000000;}  //	Start Hott Cycle
else{GIMSK |= 1<<PCIE0;}		// FemdByte ausspucken > RX aktivieren
}






